package info.kamino.Addressbook;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

/**
 * This class implements the GUI of JavAddressbook
 * @author Tobias Hergenroether
 * @version 0.1
 */
public class Addressbook extends JFrame implements ActionListener, ListSelectionListener
{
  private JTextField name;
  private JTextField surname;
  private JTextField street;
  private JTextField zip;
  private JTextField city;
  private JTextField telephone;
  private JTextField mobil;
  private JTextField email;
  private JTextField url;
  
  private JList addresses;
  
  private JMenu menuFile;
  private JMenu menuHelp;
  
  private JMenuItem menuEntriesExport;
  private JMenuItem menuEntriesImport;
  private JMenuItem menuEntriesExit;
  private JMenuItem menuEntriesAbout;
  
  AddressbookList Addresslist = new AddressbookList();
  private String keyCurrentObject;
  
  private JButton newEntry;
  private JButton editEntry;
  private JButton deleteEntry;
  
  /**
   * This is the constructor, which creates the frame and places the components
   * @param void
   * @throws nichts
   * @return void
   */
  public Addressbook(){
    super("JavAdressbuch v0.1");
    
    AddressbookSplash splash = new AddressbookSplash();
    splash.setVisible(true);
    
    splash.setLoadingModule("Customizing \"Look and Feel\"...");
    
    setLookAndFeel();
    
    splash.setProgress(20);
    splash.setLoadingModule("Initializing variables...");
    
    initMemberVariables();
    splash.setProgress(40);
    splash.setLoadingModule("Generating menu...");
    
    setLayout(null);
    
    generateMenu();
    splash.setProgress(60);
    splash.setLoadingModule("Importing data...");
    
    // Import data from standard data file (name needs to be defined)
    // importData();
    splash.setProgress(80);
    splash.setLoadingModule("Placing components on GUI...");
    
    placeComponents();
    splash.setProgress(100);
    
    //splash.dispose();
    
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  }
  /**
   * The main method is the entry point to the application
   * A Addressbook object will be created and starts the application
   * @param String[]
   * @throws nothing
   * @return void
   */
  public static void main(String[] args)
  {
    Addressbook main = new Addressbook();
    main.setSize(640, 450);
    main.setLocation(50, 100);
    main.setVisible(true);
    
  }
  
  /**
   * The method setLookAndFeel() defines the look of the  GUI.
   * The standard look of the current operating system will be defined.
   * @param void
   * @throws nothing
   * @return void
   */
  public void setLookAndFeel(){
    System.out.println("Defining Look and Feel");
    try{
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }catch (Exception e){
      System.err.println("Can't use the system Look and Feel: " + e);
    }
  }
  
  /**
   * The method initMemberVariables() initializes all member variables of this class
   * @param void
   * @throws nothing
   * @return void
   */
  private void initMemberVariables(){
    System.out.println("Initializing of variables");
    
    menuFile = new JMenu("File");
    menuEntriesExport = new JMenuItem("Export");
    menuEntriesImport = new JMenuItem("Import");
    menuEntriesExit = new JMenuItem("Exit");
        
    menuHelp = new JMenu("Help");
    menuEntriesAbout = new JMenuItem("About");
    
    name = new JTextField(20);
    name.setToolTipText("The name of this contact");
    name.setEditable(false);
    
    surname = new JTextField(20);
    surname.setToolTipText("The surname of this contact");
    surname.setEditable(false);
    
    street = new JTextField(20);
    street.setToolTipText("The street this contact is living");
    street.setEditable(false);
    
    zip = new JTextField(20);
    zip.setToolTipText("The zip code this contacts address");
    zip.setEditable(false);
    
    city = new JTextField(20);
    city.setToolTipText("The city of this contact");
    city.setEditable(false);
    
    telephone = new JTextField(20);
    telephone.setToolTipText("The telephone number of this contact");
    telephone.setEditable(false);
    
    mobil = new JTextField(20);
    mobil.setToolTipText("The cellphone number of this contact");
    mobil.setEditable(false);
    
    email = new JTextField(20);
    email.setToolTipText("The email address of this contact");
    email.setEditable(false);
    
    url = new JTextField(20);
    url.setToolTipText("The URL of this contact");
    url.setEditable(false);

    addresses = new JList();
    addresses.setToolTipText("List of all contacts sort by name");
    addresses.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    addresses.setLayoutOrientation(JList.VERTICAL);
    addresses.addListSelectionListener(this);
    
    newEntry = new JButton("New");
    newEntry.setToolTipText("Press this Button to create a new entry");
    newEntry.setMnemonic('n');
    newEntry.addActionListener(this);
    
    editEntry = new JButton("Edit");
    editEntry.setToolTipText("Press this Button to edit the selected entry");
    editEntry.setMnemonic('e');
    editEntry.addActionListener(this);
    
    deleteEntry = new JButton("Delete");
    deleteEntry.setToolTipText("Press this Button to delete the selected entry");
    deleteEntry.setMnemonic('d');
    deleteEntry.addActionListener(this); 
  }
  
 
  /**
   * The method generateMenu() creates the menu and places it in the GUI
   * @param void
   * @throws nothing
   * @return void
   */
  private void generateMenu(){
    System.out.println("Creating menu");
    
    JMenuBar bar = new JMenuBar();
      
    menuEntriesExport.addActionListener(this);
    menuFile.add(menuEntriesExport);
    menuEntriesImport.addActionListener(this);
    menuFile.add(menuEntriesImport);
    menuEntriesExit.addActionListener(this);
    menuFile.add(menuEntriesExit);
    bar.add(menuFile);
    
    menuEntriesAbout.addActionListener(this);
    menuHelp.add(menuEntriesAbout);
    bar.add(menuHelp);
    
    this.setJMenuBar(bar);
    
  }
  
  /**
   * The method placeComponents() places the components at the GUI
   * @param void
   * @throws nothing
   * @return void
   */
  private void placeComponents(){
    System.out.println("Placing Components");
    
    JLabel listLabel = new JLabel("Contact list:");
    
    listLabel.setBounds(20, 20, 200, 20);
    getContentPane().add(listLabel);
    
    JScrollPane scrollPane = new JScrollPane(addresses,
        ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPane.setBounds(20, 50, 200, 277);
    
    getContentPane().add(scrollPane);
    
    JLabel contactLabel = new JLabel("Contact information: ");
    contactLabel.setBounds(240, 20, 380, 20);
    contactLabel.setLabelFor(addresses);
    getContentPane().add(contactLabel);
    
    JLabel nameLabel = new JLabel("Name: ");
    nameLabel.setBounds(240, 50, 80, 20);
    nameLabel.setLabelFor(name);
    getContentPane().add(nameLabel);
    
    name.setBounds(320, 49, 300, 22);
    getContentPane().add(name);
    
    JLabel surnameLabel = new JLabel("Surname: ");
    surnameLabel.setBounds(240, 82, 80, 20);
    surnameLabel.setLabelFor(surname);
    getContentPane().add(surnameLabel);
    
    surname.setBounds(320, 81, 300, 22);
    getContentPane().add(surname);
    
    JLabel streetLabel = new JLabel("Street: ");
    streetLabel.setBounds(240, 114, 80, 20);
    streetLabel.setLabelFor(street);
    getContentPane().add(streetLabel);
    
    street.setBounds(320, 113, 300, 22);
    getContentPane().add(street);
    
    JLabel zipLabel = new JLabel("Zip: ");
    zipLabel.setBounds(240, 146, 80, 20);
    zipLabel.setLabelFor(zip);
    getContentPane().add(zipLabel);
    
    zip.setBounds(320, 145, 300, 22);
    getContentPane().add(zip);
    
    JLabel cityLabel = new JLabel("City: ");
    cityLabel.setBounds(240, 178, 80, 20);
    cityLabel.setLabelFor(city);
    getContentPane().add(cityLabel);
    
    city.setBounds(320, 177, 300, 22);
    getContentPane().add(city);
    
    JLabel telephoneLabel = new JLabel("Telephone: ");
    telephoneLabel.setBounds(240, 210, 80, 20);
    telephoneLabel.setLabelFor(telephone);
    getContentPane().add(telephoneLabel);
    
    telephone.setBounds(320, 209, 300, 22);
    getContentPane().add(telephone);
    
    JLabel mobilLabel = new JLabel("Mobil: ");
    mobilLabel.setBounds(240, 242, 80, 20);
    mobilLabel.setLabelFor(mobil);
    getContentPane().add(mobilLabel);
    
    mobil.setBounds(320, 241, 300, 22);
    getContentPane().add(mobil);
    
    JLabel emailLabel = new JLabel("eMail: ");
    emailLabel.setBounds(240, 274, 80, 20);
    emailLabel.setLabelFor(email);
    getContentPane().add(emailLabel);
    
    email.setBounds(320, 273, 300, 22);
    getContentPane().add(email);
    
    JLabel urlLabel = new JLabel("URL: ");
    urlLabel.setBounds(240, 306, 80, 20);
    urlLabel.setLabelFor(url);
    getContentPane().add(urlLabel);
    
    url.setBounds(320, 305, 300, 22);
    getContentPane().add(url);
    
    newEntry.setBounds(80, 360, 80, 30);
    editEntry.setBounds(280, 360, 80, 30);
    deleteEntry.setBounds(480, 360, 80, 30);
    
    getContentPane().add(newEntry);
    getContentPane().add(editEntry);
    getContentPane().add(deleteEntry);
    
  }
  
  /**
   * The method actionPerformed() implements the interface ActionListener and will be used
   * to handle the button activities 
   * @param ActionEvent
   * @throws nothing
   * @return void
   */
  public void actionPerformed(ActionEvent event){
    Object source = event.getSource();
    
    if (source == newEntry){
      
      handleNewEntry();
           
    }else if (source == editEntry){
      
      handleEditEntry();
      
    }else if (source == deleteEntry){
      
      handleDeleteEntry();
      
    }else if (source == menuEntriesExport){
      // Open JFileChooser
      JFileChooser dateiAuswahl = new JFileChooser("$HOME/");
      int status = dateiAuswahl.showSaveDialog(this);
      
      // Handle JFileChooser choice
      if (status == JFileChooser.APPROVE_OPTION){
        
        // Calling export function with the file location to export to
        exportData(dateiAuswahl.getSelectedFile().toString());
        
      }
      
    }else if (source == menuEntriesImport){
      // Open JFileChooser
      JFileChooser dateiAuswahl = new JFileChooser("$HOME/");
      int status = dateiAuswahl.showOpenDialog(this);
      
      // Handle JFileChooser choice
      if (status == JFileChooser.APPROVE_OPTION){
        
        // Calling import function with the file location to import from
        importData(dateiAuswahl.getSelectedFile().toString());
      
      }
      
      updateList();
      
    }else if (source == menuEntriesExit){
      
      // Saving the current list to the standard data file
      
      
      System.exit(0);
      
    }else if (source == menuEntriesAbout){
      
      JOptionPane.showMessageDialog(
          this,
          "I'm working in this",
          "About Dialog",
          JOptionPane.PLAIN_MESSAGE
          );
    } 
  }
  
  /**
   * The method valueCanged() implements the interface ListSelectionListener and
   * will be used to handle the JList selections 
   * @param ListSelectionEvent
   * @throws nothing
   * @return void
   */
  public void valueChanged(ListSelectionEvent lse){
    JList elements = (JList) lse.getSource();
    Object selection = elements.getSelectedValue();
    
    if (selection != null){
      System.out.println("Saving the current key");
      
      keyCurrentObject = selection.toString();
      
      System.out.println("Getting addressbook entry by key");
      AddressbookEntry temp = (AddressbookEntry) AddressbookList.get(selection.toString());
      
      if (temp != null){
        System.out.println("Filling fields");
        name.setText(temp.getName());
        surname.setText(temp.getSurname());
        street.setText(temp.getStreet());
        zip.setText(temp.getZip());
        city.setText(temp.getCity());
        telephone.setText(temp.getTelephone());
        mobil.setText(temp.getMobil());
        email.setText(temp.getEmail());
        url.setText(temp.getUrl());
      }else{
        System.err.println("Object not found!");
      }
    }
  }
  
  /**
   * The method resetTextFields() resets the content of each JTextField 
   * @param void
   * @throws nothing
   * @return void
   */
  public void resetTextFields(){
    name.setText("");
    surname.setText("");
    street.setText("");
    zip.setText("");
    city.setText("");
    telephone.setText("");
    mobil.setText("");
    email.setText("");
    url.setText("");
  }
  
  /**
   * The method updateList() refreshes the JList with all entries 
   * @param void
   * @throws nothing
   * @return void
   */
  public void updateList(){
    System.out.println("Refreshes the JList with all entries");
    String[] allKeys = (String[])AddressbookList.getKeys();
    Arrays.sort(allKeys);
    addresses.setListData(allKeys);
  }
  
  /**
   * The method handleDeleteEntry() handles the event of creating an new AddressbookEntry 
   * @param void
   * @throws nothing
   * @return void
   */
  public void handleNewEntry(){
    System.out.println("Opening Frame to create a new entry");
    new AddressbookNew(this);
  }
  
  /**
   * The method handleDeleteEntry() handles the event of editing an AddressbookEntry 
   * @param void
   * @throws nothing
   * @return void
   */
  public void handleEditEntry(){
    if (keyCurrentObject != null){
      
      new AddressbookEdit(this, keyCurrentObject);
      resetTextFields();
    }
    
    keyCurrentObject = null;
    
    updateList();
  }
  
  /**
   * The method handleDeleteEntry() handles the event of deleting an AddressbookEntry 
   * @param void
   * @throws nothing
   * @return void
   */
  public void handleDeleteEntry(){
    int response;
    if (keyCurrentObject != null){
      
      System.out.println("Viewing Deleting Confirmation Dialog");
      
      response = JOptionPane.showOptionDialog(
          this,
          "Are you sure to delete this entry?",
          "Are you sure?",
          JOptionPane.OK_CANCEL_OPTION,
          JOptionPane.QUESTION_MESSAGE,
          null,
          null,
          null);
      
      if (response == JOptionPane.OK_OPTION){
        System.out.println("Deleting entry");
        
        if ( AddressbookList.remove(keyCurrentObject) == true){
          
          resetTextFields();
          
        }else{
          System.err.println("" + keyCurrentObject + " could not be deleted!");
        }
      }else {
        
        resetTextFields();
        
      }
    }
    
    keyCurrentObject = null;
    
    updateList();
  }
  
  /**
   * The method importData() imports contact informations from given location
   * @param void
   * @throws nothing
   * @return void
   */
  private void importData(String importFile){
    
    // open file to read out of it
    try {
      FileReader importStream = new FileReader(importFile);
      
      BufferedReader importBuffer = new BufferedReader(importStream);
      
      // Reading one line out of the import file
      String readingString = importBuffer.readLine();
      while( readingString != null){
        
        StringTokenizer sTokenizer = new StringTokenizer(readingString, ";");
        
        // Creating AddressbookEntry object with the contents of the tokens
        AddressbookEntry data = new AddressbookEntry();
        int i = 1;
        while(sTokenizer.hasMoreTokens()){
          
          String temp = sTokenizer.nextToken();
          
          // Removing the blank at the end of the string, which was added during export
          if ( temp.endsWith(" ")){
            temp = temp.substring(0, (temp.length()-1));
          }
          
          switch (i++){
          case 1:   data.setName(temp);
                    break;
          case 2:   data.setSurname(temp);
                    break;
          case 3:   data.setStreet(temp);
                    break;
          case 4:   data.setZip(temp);
                    break;
          case 5:   data.setCity(temp);
                    break;
          case 6:   data.setTelephone(temp);
                    break;
          case 7:   data.setMobil(temp);
                    break;
          case 8:   data.setEmail(temp);
                    break;
          case 9:   data.setUrl(temp);
                    break;
          default:  System.err.println("Import error - " +
                    "Token can't be associated with AddressbookEntry member variable.");
                    break;
          }
        }
        // Adding the new AddressbookEntry object to list
        AddressbookList.add("" + data.getName() + ", " + data.getSurname(), data);
        
        // Getting next line out of import File
        readingString = importBuffer.readLine();
      }
    
      importStream.close();
    }
    catch (IOException e){
      JOptionPane.showMessageDialog(
          this,
          "An error occured while importing the Addressbook entries!",
          "Import Error",
          JOptionPane.ERROR_MESSAGE
          );
    } 
  }
  
  
  /**
   * The method exportData() exports contact informations to given location
   * @param String exportFile
   * @throws nothing
   * @return void
   */
  public void exportData (String exportFile){
    
    try {
      FileWriter exportStream = new FileWriter(exportFile);
      
      BufferedWriter exportBuffer = new BufferedWriter(exportStream);
      
      // Getting all keys to iterate throw Hashtable
      String[] allKeys = (String[])AddressbookList.getKeys();
      
      for(int i = 0; i < allKeys.length; i++){
        AddressbookEntry temp = AddressbookList.get(allKeys[i]);
        
        // Creating one Line with all data seperates by ' ;'
        String writingString = temp.getName() + " ;" + temp.getSurname() + " ;" + 
                                temp.getStreet() + " ;" + temp.getZip() + " ;" + 
                                temp.getCity() + " ;" + temp.getTelephone() + " ;" + 
                                temp.getMobil() + " ;" + temp.getEmail() + " ;" + 
                                temp.getUrl();
        // Writing one Addressbook object into file plus a page break
        exportBuffer.write(writingString);
        exportBuffer.newLine();
      }
      exportBuffer.close();
    }
    catch (IOException e){
      JOptionPane.showMessageDialog(
          this,
          "An error occured while exporting the Addressbook entries!",
          "Export Error",
          JOptionPane.ERROR_MESSAGE
          );
    }
  }
}
