package info.kamino.Addressbook;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class implements the GUI to create a new addressbook entry and
 * extends the Addressbook class
 * @author Tobias Hergenroether
 * @version 0.1
 */
public class AddressbookNew extends AddressbookDialog implements ActionListener
{
  private JButton save;
  private JButton reset;
  private JButton abort;

  private AddressbookEntry data;
  private Addressbook parentFrame;

  /**
   * The constructor of the frame, which creates the frame and places the components
   * @param (Adressbook) parent
   * @throws nothing
   * @return void
   */
  public AddressbookNew(Addressbook callingObject)
  {
    super("Create new entry");
    setBounds(50, 50, 380, 400);
    setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

    parentFrame = callingObject;
    parentFrame.setLookAndFeel();

    getContentPane().setLayout(null);
    
    initMemberVariables();

    placeComponents();
    
    setVisible(true);
  }
  
  /**
   * The method initMemberVariables() initializes all member variables of this class
   * @param void
   * @throws nothing
   * @return void
   */
  private void initMemberVariables(){
    
    save = new JButton("Save");
    save.addActionListener(this);
    
    reset = new JButton("Reset");
    reset.addActionListener(this);
    
    abort = new JButton("Abort");
    abort.addActionListener(this);
    
  }
  
  /**
   * The method placeComponents() creates and places the components on content pane
   * Some components belong to the super class of AddressbookNew class.
   * @param void
   * @throws nichts
   * @return void
   */
  private void placeComponents()
  {
    System.out.println("Placing Compnents (AdressbuchNeu)");
    
    JLabel headline = new JLabel("Please insert address information");
    headline.setBounds(20, 5, 340, 20);
    getContentPane().add(headline);
    
    JLabel mandatory = new JLabel("Fields marked with an asterisk are mandatory");
    mandatory.setBounds(20, 25, 340, 20);
    getContentPane().add(mandatory);

    JLabel nameLabel = new JLabel("Name*: ");
    nameLabel.setBounds(20, 52, 80, 20);
    nameLabel.setLabelFor(name);
    getContentPane().add(nameLabel);
    
    name.setBounds(100, 51, 260, 22);
    getContentPane().add(name);

    JLabel surnameLabel = new JLabel("Surname*: ");
    surnameLabel.setBounds(20, 82, 80, 20);
    surnameLabel.setLabelFor(surname);
    getContentPane().add(surnameLabel);

    surname.setBounds(100, 81, 260, 22);
    getContentPane().add(surname);

    JLabel streetLabel = new JLabel("Street: ");
    streetLabel.setBounds(20, 112, 80, 20);
    streetLabel.setLabelFor(street);
    getContentPane().add(streetLabel);

    street.setBounds(100, 111, 260, 22);
    getContentPane().add(street);

    JLabel zipLabel = new JLabel("ZIP: ");
    zipLabel.setBounds(20, 141, 80, 20);
    zipLabel.setLabelFor(zip);
    getContentPane().add(zipLabel);

    zip.setBounds(100, 140, 260, 22);
    getContentPane().add(zip);

    JLabel cityLabel = new JLabel("City: ");
    cityLabel.setBounds(20, 171, 80, 20);
    cityLabel.setLabelFor(city);
    getContentPane().add(cityLabel);

    city.setBounds(100, 170, 260, 22);
    getContentPane().add(city);

    JLabel telephoneLabel = new JLabel("Telephone: ");
    telephoneLabel.setBounds(20, 201, 80, 20);
    telephoneLabel.setLabelFor(telephone);
    getContentPane().add(telephoneLabel);

    telephone.setBounds(100, 200, 260, 22);
    getContentPane().add(telephone);

    JLabel mobilLabel = new JLabel("Mobil: ");
    mobilLabel.setBounds(20, 231, 80, 20);
    mobilLabel.setLabelFor(mobil);
    getContentPane().add(mobilLabel);

    mobil.setBounds(100, 230, 260, 22);
    getContentPane().add(mobil);

    JLabel emailLabel = new JLabel("eMail: ");
    emailLabel.setBounds(20, 261, 80, 20);
    emailLabel.setLabelFor(email);
    getContentPane().add(emailLabel);

    email.setBounds(100, 260, 260, 22);
    getContentPane().add(email);

    JLabel urlLabel = new JLabel("URL: ");
    urlLabel.setBounds(20, 291, 80, 20);
    urlLabel.setLabelFor(url);
    getContentPane().add(urlLabel);

    url.setBounds(100, 290, 260, 22);
    getContentPane().add(url);
    
    save.setBounds(20, 330, 75, 30);
    getContentPane().add(save);
    
    reset.setBounds(152, 330, 75, 30);
    getContentPane().add(reset);
    
    abort.setBounds(285, 330, 75, 30);
    getContentPane().add(abort);
    
  }

  /**
   * The method actionPerformed() implements the interface ActionListener an will be used
   * to handle the button activities 
   * @param ActionEvent
   * @throws nothing
   * @return void
   */
  public void actionPerformed(ActionEvent event)
  {
    Object source = event.getSource();
    if (source == save)
    {
      // Checking for filled mandatory fields
      if (!name.getText().contentEquals("") && !surname.getText().contentEquals("")){
        
        System.out.println("Storing entered data in a Addressbook object");
  
        data = new AddressbookEntry(name.getText(), surname.getText(), street.getText(), zip.getText(), city.getText(),
            telephone.getText(), mobil.getText(), email.getText(), url.getText());
  
        System.out.println("Creating Key");
        String key = "" + name.getText() + ", " + surname.getText();
  
        System.out.println("Adding new entry to Hashtable");
        if (AddressbookList.add(key, data)){
          System.out.println("Entry successfully added");
        } else {
          System.err.println("Adding entry to Hashtable failed");
        }
  
        parentFrame.updateList();
  
        dispose();
      }else{
        JOptionPane.showMessageDialog(this,
            "Please enter a name and a surname!");
      }
    }
    else if (source == reset)
    {
      System.out.println("Reset Textfields");

      name.setText("");
      surname.setText("");
      street.setText("");
      zip.setText("");
      city.setText("");
      telephone.setText("");
      mobil.setText("");
      email.setText("");
      url.setText("");

    }
    else if (source == abort)
    {
      System.out.println("Creation of new Addressbook-Entry aborted");
      dispose();
    }
  }
}
